from fastapi import APIRouter
from processrepo.ProcessRunProfile import ProcessRunProfile
from processrepo.repository.ProcessRunProfileRepository import ProcessRunProfileRepository

from apiautomata.holder.ItemHolder import ItemHolder

router = APIRouter()


@router.get('/process/{market}/{name}', response_model=ProcessRunProfile)
async def get_process_run_profile(market, name):
    process_run_profile_repository = ItemHolder.get_entity(ProcessRunProfileRepository)
    return process_run_profile_repository.retrieve(name, market)


@router.post('/process/{market}/{name}', response_model=bool)
async def enable_proces(market, name, enable: bool = True):
    process_run_profile_repository = ItemHolder.get_entity(ProcessRunProfileRepository)
    process_run_profile = process_run_profile_repository.retrieve(name, market)
    process_run_profile.enabled = enable
    process_run_profile_repository.store(process_run_profile)
    return process_run_profile.enabled



from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware

from apiautomata.routes import home, proxy
from apiautomata.routes.exchange import instrument_exchange, exchange_rate
from apiautomata.routes.missing import missing
from apiautomata.routes.process import process_enablement, process_status, process_invoke
from apiautomata.routes.transform import exchange_transform, trade_transform

app = FastAPI()

app.include_router(home.router)
app.include_router(proxy.router)
app.include_router(missing.router)
app.include_router(instrument_exchange.router)
app.include_router(exchange_transform.router)
app.include_router(trade_transform.router)
app.include_router(exchange_rate.router)
app.include_router(process_enablement.router)
app.include_router(process_status.router)
app.include_router(process_invoke.router)

origins = ['*']

app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,
    allow_credentials=True,
    allow_methods=['*'],
    allow_headers=['*'],
)


@app.on_event('startup')
async def startup_event():
    print('Starting...')


@app.on_event('shutdown')
async def shutdown_event():
    print('Stopping...')



from typing import List

from core.exchange.InstrumentExchange import InstrumentExchange
from exchangerepo.repository.InstrumentExchangeRepository import InstrumentExchangeRepository
from fastapi import APIRouter

from apiautomata.holder.ItemHolder import ItemHolder

router = APIRouter()


@router.get('/exchange/instrument', response_model=List[InstrumentExchange])
async def get_all_exchange_instruments():
    instrument_exchange_repository = ItemHolder.get_entity(InstrumentExchangeRepository)
    holder = instrument_exchange_repository.retrieve()
    return holder.get_all()


@router.put('/exchange/instrument')
async def create_exchange_instrument(instrument_exchange: InstrumentExchange):
    instrument_exchange_repository = ItemHolder.get_entity(InstrumentExchangeRepository)
    instrument_exchange_repository.create(instrument_exchange)
    return instrument_exchange


@router.post('/exchange/instrument')
async def update_exchange_instrument(instrument_exchange: InstrumentExchange):
    instrument_exchange_repository = ItemHolder.get_entity(InstrumentExchangeRepository)
    instrument_exchange_repository.update(instrument_exchange)
    return instrument_exchange


@router.delete('/exchange/instrument')
async def delete_exchange_instrument(instrument_exchange: InstrumentExchange):
    instrument_exchange_repository = ItemHolder.get_entity(InstrumentExchangeRepository)
    instrument_exchange_repository.delete(instrument_exchange)
    return instrument_exchange



from typing import List

from fastapi import APIRouter
from missingrepo.Missing import Missing
from missingrepo.repository.MissingRepository import MissingRepository

from apiautomata.holder.ItemHolder import ItemHolder

router = APIRouter()


@router.get('/missing', response_model=List[Missing])
async def get_all_missing():
    missing_repository = ItemHolder.get_entity(MissingRepository)
    return missing_repository.retrieve()


@router.delete('/missing')
async def delete_missing(missing: Missing):
    missing_repository = ItemHolder.get_entity(MissingRepository)
    missing_repository.delete(missing)
    return 'SUCCESS'



from typing import List

from fastapi import APIRouter
from tradetransformrepo.TradeTransform import TradeTransform
from tradetransformrepo.repository.TradeTransformRepository import TradeTransformRepository

from apiautomata.holder.ItemHolder import ItemHolder

router = APIRouter()


@router.get('/transform/trade', response_model=List[TradeTransform])
async def get_all_trade_transforms():
    trade_transform_repository = ItemHolder.get_entity(TradeTransformRepository)
    return trade_transform_repository.retrieve()


@router.put('/transform/trade')
async def create_trade_transform(trade_transform: TradeTransform):
    trade_transform_repository = ItemHolder.get_entity(TradeTransformRepository)
    trade_transform_repository.create(trade_transform)
    return trade_transform


@router.post('/transform/trade')
async def update_trade_transform(trade_transform: TradeTransform):
    trade_transform_repository = ItemHolder.get_entity(TradeTransformRepository)
    trade_transform_repository.update(trade_transform)
    return trade_transform


@router.delete('/transform/trade')
async def delete_trade_transform(trade_transform: TradeTransform):
    trade_transform_repository = ItemHolder.get_entity(TradeTransformRepository)
    trade_transform_repository.delete(trade_transform)
    return trade_transform



from fastapi import APIRouter, Response, status

from apiautomata.holder.ItemHolder import ItemHolder
from apiautomata.process.ProcessInvoker import ProcessInvoker

router = APIRouter()


@router.post('/process/invoke', status_code=200)
async def invoke_process(process_name: str, response: Response):
    process_run_command = ItemHolder.get('process-run-command')
    process_invoker = ProcessInvoker(process_run_command, process_name)
    result = process_invoker.invoke_process()
    response.status_code = status.HTTP_406_NOT_ACCEPTABLE if result.startswith('error:') else status.HTTP_201_CREATED
    return result



import asyncio
from fastapi import APIRouter

from apiautomata.proxify.Proxify import Proxify

router = APIRouter()


@router.get('/proxy')
async def proxy(url: str = None):
    proxify = Proxify(url)

    tasks = []
    task = asyncio.create_task(Proxify.get_end_point_status(proxify))
    tasks.append(task)

    await asyncio.gather(*tasks)

    return proxify



import logging

from cache.holder.RedisCacheHolder import RedisCacheHolder
from cache.provider.RedisCacheProviderWithTimeSeries import RedisCacheProviderWithTimeSeries
from core.arguments.command_line_arguments import option_arg_parser
from logger.ConfigureLogger import ConfigureLogger
from metainfo.MetaInfo import MetaInfo

from apiautomata.server.AutomataAPIServer import AutomataAPIServer


def start():
    ConfigureLogger()

    meta_info = MetaInfo('persuader-technology-automata-api')

    command_line_arg_parser = option_arg_parser(meta_info)
    args = command_line_arg_parser.parse_args()

    log = logging.getLogger('Automata API')
    log.info(f'Automata API starting with OPTIONS {args.options}')

    RedisCacheHolder(args.options, held_type=RedisCacheProviderWithTimeSeries)

    server = AutomataAPIServer(args.options)
    server.run()


if __name__ == '__main__':
    start()



import subprocess


class ProcessInvoker:

    def __init__(self, process_run_command, process_name):
        self.process_run_command = process_run_command
        self.process_name = process_name

    def invoke_process(self):
        try:
            command = self.build_command()
            pid = subprocess.Popen([command], shell=True).pid
            return f'process invoked: {pid}'
        except BaseException as ex:
            return f'error: {ex}'

    def build_command(self):
        return self.process_run_command.replace('{process-name}', self.process_name)



import aiohttp
from coreutility.date_utility import get_utc_timestamp


class Proxify:
    cached_calls = {}

    def __init__(self, url):
        self.url = url
        self.status = None
        self.instant = get_utc_timestamp()

    @classmethod
    async def get_end_point_status(cls, obj):
        if obj.url in Proxify.cached_calls.keys():
            obj.status = Proxify.cached_calls[obj.url]
        else:
            async with aiohttp.ClientSession() as session:
                async with session.get(obj.url) as response:
                    response_status = response.status
                    obj.status = response_status
                    Proxify.cached_calls[obj.url] = response_status



from fastapi import APIRouter
from processrepo.Process import Process
from processrepo.repository.ProcessRepository import ProcessRepository

from apiautomata.holder.ItemHolder import ItemHolder

router = APIRouter()


@router.get('/process/status/{market}/{name}', response_model=Process)
async def get_process_status(market, name):
    process_repository = ItemHolder.get_entity(ProcessRepository)
    return process_repository.retrieve(name, market)



from coreutility.date_utility import get_utc_timestamp
from fastapi import APIRouter

from apiautomata.holder.ItemHolder import ItemHolder

router = APIRouter()


@router.get('/')
async def index():
    return {'api': 'Automata API', 'version': ItemHolder.get('version'), 'instant': get_utc_timestamp()}




from typing import List

from exchangetransformrepo.ExchangeTransform import ExchangeTransform
from exchangetransformrepo.repository.ExchangeTransformRepository import ExchangeTransformRepository
from fastapi import APIRouter

from apiautomata.holder.ItemHolder import ItemHolder

router = APIRouter()


@router.get('/transform/exchange', response_model=List[ExchangeTransform])
async def get_all_exchange_transforms():
    exchange_transform_repository = ItemHolder.get_entity(ExchangeTransformRepository)
    return exchange_transform_repository.retrieve()


@router.put('/transform/exchange')
async def create_exchange_transform(exchange_transform: ExchangeTransform):
    exchange_transform_repository = ItemHolder.get_entity(ExchangeTransformRepository)
    exchange_transform_repository.create(exchange_transform)
    return exchange_transform


@router.post('/transform/exchange')
async def update_exchange_transform(exchange_transform: ExchangeTransform):
    exchange_transform_repository = ItemHolder.get_entity(ExchangeTransformRepository)
    exchange_transform_repository.update(exchange_transform)
    return exchange_transform


@router.delete('/transform/exchange')
async def delete_exchange_transform(exchange_transform: ExchangeTransform):
    exchange_transform_repository = ItemHolder.get_entity(ExchangeTransformRepository)
    exchange_transform_repository.delete(exchange_transform)
    return exchange_transform



import logging

import uvicorn
from exchangerepo.repository.ExchangeRateRepository import ExchangeRateRepository
from exchangerepo.repository.InstrumentExchangeRepository import InstrumentExchangeRepository
from exchangetransformrepo.repository.ExchangeTransformRepository import ExchangeTransformRepository
from missingrepo.repository.MissingRepository import MissingRepository
from processrepo.repository.ProcessRepository import ProcessRepository
from processrepo.repository.ProcessRunProfileRepository import ProcessRunProfileRepository
from tradetransformrepo.repository.TradeTransformRepository import TradeTransformRepository

from apiautomata.holder.ItemHolder import ItemHolder


class AutomataAPIServer:

    def __init__(self, options):
        self.log = logging.getLogger('AutomataAPIServer')
        self.options = options
        self.host = options['API_SERVER_HOST']
        self.port = options['API_SERVER_PORT']
        self.init_dependencies()

    def init_dependencies(self):
        self.log.info('Initializing dependencies')
        item_holder = ItemHolder()
        item_holder.add(self.options['VERSION'], 'version')
        item_holder.add(self.options['PROCESS_RUN_COMMAND'], 'process-run-command')
        item_holder.add_entity(MissingRepository(self.options))
        item_holder.add_entity(InstrumentExchangeRepository(self.options))
        item_holder.add_entity(ExchangeTransformRepository(self.options))
        item_holder.add_entity(TradeTransformRepository(self.options))
        item_holder.add_entity(ExchangeRateRepository(self.options))
        item_holder.add_entity(ProcessRunProfileRepository(self.options))
        item_holder.add_entity(ProcessRepository(self.options))

    def run(self):
        self.log.info('Running')
        uvicorn.run('apiautomata.API:app', host=self.host, port=self.port, access_log=False)



from typing import Optional, Type, TypeVar

from apiautomata.holder.Items import Items

T = TypeVar('T')


class ItemHolder:
    __instance: Optional[Items] = None

    def __new__(cls):
        if cls.__instance is None:
            cls.__instance = Items()
        return cls.__instance

    @staticmethod
    def get(name):
        return ItemHolder.__instance.get(name)

    @staticmethod
    def get_entity(cls: Type[T]) -> T:
        return ItemHolder.__instance.get_entity(cls)



from typing import TypeVar, Type

T = TypeVar('T')


class Items:

    def __init__(self):
        self.items = {}

    def add(self, value, name):
        self.items[name] = value

    def get(self, name):
        return self.items[name]

    def add_entity(self, entity: T):
        name = type(entity).__name__
        self.items[name] = entity

    def get_entity(self, cls: Type[T]) -> T:
        name = cls.__name__
        return self.items[name]



from typing import Dict

from core.exchange.InstrumentExchange import InstrumentExchange
from exchangerepo.repository.ExchangeRateRepository import ExchangeRateRepository
from fastapi import APIRouter

from apiautomata.holder.ItemHolder import ItemHolder

router = APIRouter()


@router.get('/exchange/rate/{instrument_from}/{instrument_to}', response_model=Dict)
async def get_exchange_rate(instrument_from, instrument_to):
    instrument_exchange = InstrumentExchange(instrument_from, instrument_to)
    exchange_rate_repository = ItemHolder.get_entity(ExchangeRateRepository)
    instant_rate = exchange_rate_repository.retrieve_latest(instrument_exchange)
    if instant_rate is not None:
        return {'rate': str(instant_rate.rate), 'instant': instant_rate.instant, 'inverse': str(instant_rate.rate.invert())}
    return {}
